extends Node2D

@export var k = 0.015
@export var d = 0.03
@export var spread = 0.0006

var springs = []
var passes = 10

@onready var water_spring = preload("res://Water/water_spring.tscn")
@export var distance_between_springs = 64
@export var spring_number = 10

@export var depth = 400
var target_height = global_position.y
var bottom = target_height + depth
@onready var water_polygon = $Water_Polygon

@onready var water_border = $Water_Border
@export var border_thickness = 1

func _ready():
	$ColorRect.visible = false
	water_border.width = border_thickness
	
	for i in range(spring_number):
		var x_pos = distance_between_springs * i
		var w = water_spring.instantiate()
		w.initialize(x_pos, 0, i)
		w.connect("splash", splash)
		springs.append(w)
		add_child(w)
		
	splash(9, 10)
	
func _physics_process(_delta):
	for i in springs:
		i.water_update(k, d)
	var left_deltas = []
	var right_deltas = []
	
	for i in range(springs.size()):
		left_deltas.append(0)
		right_deltas.append(0)
	
	for j in range(passes):
		for i in range(springs.size()):
			if i>0:
				left_deltas[i] = (springs[i].height - springs[i-1].height) * spread
				springs[i-1].velocity += left_deltas[i]
			if i < springs.size()-1:
				right_deltas[i] = (springs[i].height - springs[i+1].height) * spread
				springs[i+1].velocity += right_deltas[i]

	draw_water()
	draw_border()

func splash(i, speed):
	if i>=0 and i<= springs.size():
		springs[i].velocity += speed

func draw_water():
	var surface_points = []
	for i in springs:
		surface_points.append(i.position)
	
	var water_poly_points = surface_points
	
	water_poly_points.append(Vector2(surface_points[surface_points.size()-1].x, depth))
	water_poly_points.append(Vector2(surface_points[0].x, depth))
	
	water_poly_points = PackedVector2Array(water_poly_points)
	water_polygon.set_polygon(water_poly_points)
	update_water_uv(water_poly_points)

func update_water_uv(poly_points):
	var v0 = poly_points
	var x0 = INF
	var y0 = INF
	var x1 = -INF
	var y1 = -INF
	for v in v0:
		x0 = min(x0, v.x)
		x1 = max(x1, v.x)
		y0 = min(y0, v.y)
		y1 = max(y1, v.y)
	#var s = max(x1 - x0, y1 - y0)
	var s1 = x1 - x0
	var s2 = y1 - y0
	var dt = Vector2(s1 / 2, s2 / 2)
	var uv = PackedVector2Array([])
	for v in v0:
		uv.append(Vector2((v.x + dt.x) / s1, (v.y + dt.y) / s2))
		
	#water_polygon.set_polygon(v0)
	water_polygon.set_uv(PackedVector2Array(uv))

func draw_border():
	var curve = Curve2D.new()#.duplicate()
	var surf_points = []
	
	for i in range(springs.size()):
		surf_points.append(springs[i].position)
	
	for i in range(surf_points.size()):
		curve.add_point(surf_points[i])
	
	water_border.curve = curve
	water_border.smooth(true)
	water_border.queue_redraw()
